/* Tex 2 Hex 1.0
 *
 * sample code for the ASDK tutorial
 *
 * this code will generate errors
 * read chapter 6 for debugging info
 *
 * by Andrew Howlett
 * howlett@iosphere.net
 * June 1997
 */

#pragma pack(2)
#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>

#include "tex2hex.h"
#define Tex2HexAppID    'TxHx'   // 0x74586858 in hexadecimal
#define Tex2HexDBType   'Data'   // 0x64415441 in hexadecimal

static int StartApplication(void);
static Boolean OpenDatabase(void);
static void EventLoop(void);
static void EventLoop(void);
static void StopApplication(void);
static Boolean tex2hex(EventPtr event);
static void Convert(void);
static void ChangeBitmap(void);

FieldPtr	fieldptr_text;   // FieldPtr and DmOpenRef are defined in the PalmOS header files.
FieldPtr	fieldptr_hex;    // See if you can find where they are defined.
DmOpenRef	Tex2HexDB;
char            Tex2HexDBName[] = "Tex2HexDB";
char            hex[60];
int             WhichBitmap;

DWord  PilotMain (Word cmd, Ptr cmdPBP, Word launchFlags)
{
  int error;

  if (cmd == sysAppLaunchCmdNormalLaunch)
 {
    error = StartApplication();	// Application start code
    if (error) return error;

    EventLoop();	// Event loop

    StopApplication ();	// Application stop code
  }
  return 0;
}

static int StartApplication(void){
  int error;

  error = OpenDatabase();
  if (error) return error;

  FrmGotoForm(formID_tex2hex);
}

static Boolean OpenDatabase(void){
  UInt          index = 0;
  VoidHand      RecHandle;
  Ptr           RecPointer;
  char          nullstring = 0;

  Tex2HexDB = DmOpenDatabaseByTypeCreator(Tex2HexDBType, Tex2HexAppID, dmModeReadWrite);
  if (!Tex2HexDB) {
    if (DmCreateDatabase(0, Tex2HexDBName, Tex2HexAppID, Tex2HexDBType, false)) return 1;
    Tex2HexDB = DmOpenDatabaseByTypeCreator(Tex2HexDBType, Tex2HexAppID, dmModeReadWrite);
    RecHandle = DmNewRecord(Tex2HexDB, &index, 1);
    RecPointer = MemHandleLock(RecHandle);
    DmWrite(RecPointer, 0, &nullstring, 1);
    MemPtrUnlock(RecPointer); 
    DmReleaseRecord(Tex2HexDB, index, true);  
  }
  return 0;
}

static void EventLoop(void){
  short err;
  int formID;
  FormPtr form;
  EventType event;

  do  {
    EvtGetEvent(&event, 200);

    if (SysHandleEvent(&event)) continue;
    if (MenuHandleEvent((void *)0, &event, &err)) continue;

    if (event.eType == frmLoadEvent)    {
      formID = event.data.frmLoad.formID;
      form = FrmInitForm(formID);
      FrmSetActiveForm(form);
      switch (formID)       {
      case formID_tex2hex:
        FrmSetEventHandler(form, (FormEventHandlerPtr) tex2hex);
        break;
      }
    }
    FrmDispatchEvent(&event);
  } while(event.eType != appStopEvent);
}

static void StopApplication(void){
  FldSetTextHandle(fieldptr_text, NULL);
  DmReleaseRecord(Tex2HexDB, 0, false);
  DmCloseDatabase(Tex2HexDB);
}

static Boolean tex2hex(EventPtr event){
  FormPtr   form;
  int       handled = 0;

  switch (event->eType)   {
  case frmOpenEvent:
    form = FrmGetActiveForm();
    fieldptr_text = FrmGetObjectPtr(form, FrmGetObjectIndex(form, fieldID_text));
    FldSetTextHandle (fieldptr_text, (Handle) DmGetRecord(Tex2HexDB, 0));
    fieldptr_hex = FrmGetObjectPtr(form, FrmGetObjectIndex(form, fieldID_hex));
    FrmDrawForm(form);
    handled = 1;
    break;
    
  case ctlSelectEvent:  // A control button was pressed and released.
    if (event->data.ctlEnter.controlID== buttonID_convert)    {
      Convert();
      handled = 1;
    }
    break;

  case menuEvent:
    switch (event->data.menu.itemID)    {
    case menuitemID_convert:
      Convert();
      break;
    case menuitemID_about:
      FrmAlert(alertID_about);
      break;
    case menuitemID_copy:
      FldCopy(fieldptr_text);   // user has to select (highllight) the text to copy
      break;
    case menuitemID_paste:
      FldPaste(fieldptr_text);
      break;
    }
    handled = 1;
    break;

  case nilEvent:
    ChangeBitmap();
    handled = 1;
    break;
  }
  return handled;
}

static void Convert(void){
  char  textchar;
  int  textpos;
  int  lownybble, hinybble;
  char  hexchar;
  int  textlength;
  CharPtr  textpointer;

  textpointer = FldGetTextPtr(fieldptr_text);
  textlength = FldGetTextLength(fieldptr_text);
  if (textlength)  {
    for (textpos = 0; textpos < textlength; textpos ++)    {
      textchar = textpointer[textpos];
      lownybble = textchar % 16;
      hinybble = textchar / 16;
      if (hinybble < 10) hexchar = 48 + hinybble;
        else hexchar = 55 + hinybble;
      hex[textpos * 3] = hexchar;
      if (lownybble < 10) hexchar = 48 + lownybble;
        else hexchar = 55 + lownybble;
      hex[textpos * 3 + 1] = hexchar;
      hex[textpos * 3 + 2] = 45;
    }
    hex [textpos * 3 - 1] = 0;
    FldSetTextPtr(fieldptr_hex, hex);
    FldRecalculateField(fieldptr_hex, true);
    FrmDrawForm(FrmGetActiveForm());
  }
  else  {
    FrmAlert(alertID_errornullstring);
  }
}

static void ChangeBitmap(void)
{
  VoidHand   bitmaphandle;
  BitmapPtr  bitmap;

  WhichBitmap = (WhichBitmap + 1) % 2;
  if (WhichBitmap == 0) bitmaphandle = DmGet1Resource('Tbmp', bitmapID_text);
  else bitmaphandle = DmGetResource('Tbmp', bitmapID_hex);
  bitmap = MemHandleLock(bitmaphandle);
  WinDrawBitmap(bitmap, 20, 120);
  MemHandleUnlock(bitmaphandle);
}
